﻿using ChatObjects;
using ChatServer;
using System;
using System.Collections.Generic;
using System.Text;

namespace Homework4Server
{
	public abstract class ObjectHandler
	{
		abstract public bool handleObject(object obj, ClientHandler server);
	}

	public class StringHandler : ObjectHandler
	{
		override public bool handleObject(object obj, ClientHandler server)
		{
			bool result = obj is string;
			if (result)
			{
				server.readWriteLock.AcquireReaderLock(-1);
				try
				{
					server.clientList.ForEach(delegate (ClientHandler client)
					{
						server.Send(client.netStream, "[" + server._clientName + "] - " + obj);
					});

				}
				finally
				{
					server.readWriteLock.ReleaseReaderLock();
				}
			}
			return result;
		}
	}

	public class LoginHandler : ObjectHandler
	{
		override public bool handleObject(object obj, ClientHandler chatHandler)
		{
			bool result = obj is Login;
			if (result)
			{
				Login login = (Login)obj;
				login.isConnected = true;
				chatHandler.readWriteLock.AcquireReaderLock(-1);
				try
				{
					chatHandler.clientList.ForEach(delegate (ClientHandler client)
					{
						if (client._clientName == login.userName)
						{
							login.isConnected = false;
						}
					});
				}
				finally
				{
					chatHandler.readWriteLock.ReleaseReaderLock();
				}
				if (login.isConnected)
				{
					chatHandler.readWriteLock.AcquireReaderLock(-1);
					try
					{
						chatHandler._clientName = login.userName;
						chatHandler.clientList.ForEach(delegate (ClientHandler client)
						{
							chatHandler.Send(client.netStream, login);
						});
					}
					finally
					{
						chatHandler.readWriteLock.ReleaseReaderLock();
					}
					chatHandler.Log("Login distributed.");
				}
				else
				{
					chatHandler.Send(chatHandler.netStream, login);
					chatHandler.Log("Same username exists [" + login.userName + "].");
				}

			}
			return result;
		}
	}
	public class LogoutHandler : ObjectHandler
	{
		override public bool handleObject(object obj, ClientHandler chatHandler)
		{
			bool result = obj is Logout;
			if (result)
			{
				Logout logout = (Logout)obj;
				chatHandler.readWriteLock.AcquireReaderLock(-1);
				try
				{
					chatHandler.clientList.ForEach(delegate (ClientHandler client)
					{
						chatHandler.Send(client.netStream, logout);
					});
				}
				finally
				{
					chatHandler.readWriteLock.ReleaseReaderLock();
				}
				chatHandler.Log("Logout distributed.");
			}
			return result;
		}
	}

	public class GetUserListHandler : ObjectHandler
	{
		override public bool handleObject(object obj, ClientHandler chatHandler)
		{
			bool result = obj is GetUserList;
			if (result)
			{
				GetUserList getUserList = (GetUserList)obj;
				getUserList.list = new List<string>();
				chatHandler.readWriteLock.AcquireReaderLock(-1);
				try
				{
					chatHandler.clientList.ForEach(delegate (ClientHandler client)
					{
						getUserList.list.Add(client._clientName);

					});
				}
				finally
				{
					chatHandler.readWriteLock.ReleaseReaderLock();
				}
				chatHandler.Send(chatHandler.netStream, getUserList);
				chatHandler.Log("User list collection sent.");
			}
			return result;
		}
	}
}